/* $Id: adjint.h,v 1.1.1.1.2.1.2.2 2006/07/07 09:51:26 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef DEP_ADJINT_INCLUDED
#define DEP_ADJINT_INCLUDED

#include "depdef.h"

using namespace GenLib2;

namespace DEP
{

/** Adjustment interface base class.
 *  General definition of interface functions for solving normal equations
 *  and calculate their inverse.
 *  @author Dirk Stallmann
 */
class AdjInterface
{
public:
	/// constructor
	AdjInterface() {}
	
	/// virtual destructor
	virtual ~AdjInterface() {}
	
	/** solve normal equation system
	 *  @param atpa (IN/OUT) array with normal matrix N in the upper part
	 *  and decomposition matrix C in the lower part
	 *  @param atpl (IN) right side vector n
	 *  @param x (OUT) solution vector x
	 *  @param dec (OUT) decomposition vector d (optional)
	 *  @return info: info = 0, OK; info = -i, invalid i-th argument;
	 *  info = i, decomposition failed in row i-1
	 */
	virtual int solve(Matrix<double>& atpa, Vector<double>& atpl,
		Vector<double>& x, Vector<double>& dec) = 0;

	/** inversion of normal matrix
	 *  @param atpa (IN) array with matrix C in the lower part
	 *  @param qxx (OUT) inverse of normal matrix N
	 *  @param dec (IN) decomposition vector d (optional)
	 *  @return info: info = 0, OK; info = -i, invalid i-th argument
	 */
	virtual int inversion(Matrix<double>& atpa, Matrix<double>& qxx,
		Vector<double>& dec) = 0;
	
	/** write structure of normal matrix in PBM-format
	 *  @param os (OUT) output stream
	 *  @param atpa (IN) normal matrix N
	 */
	void write_matrix_struct(ostream& os,
		const Matrix<double>& atpa) const;
};

} // namespace DEP

#endif
