/* $Id: adjintlap.cpp,v 1.2.2.1.2.1 2005/12/12 12:27:29 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "adjintlap.h"

#ifdef GENLIB2_HAVE_LAPACK
#include "lapack.h"
#endif

namespace DEP
{

// solve normal equation system

int AdjInterfaceLapack::solve(Matrix<double>& atpa, Vector<double>& atpl,
Vector<double>& x, Vector<double>& dec)
{
	x = atpl;

#ifdef GENLIB2_HAVE_LAPACK

	return lapack_dposv(atpa, x);

#else

	cout << "ERROR: Compiled without LAPACK.\n";
	return -999;

#endif // GENLIB2_HAVE_LAPACK
}


// inversion of normal matrix

int AdjInterfaceLapack::inversion(Matrix<double>& atpa, Matrix<double>& qxx,
Vector<double>& dec)
{
#ifdef GENLIB2_HAVE_LAPACK

	qxx = atpa;
	int info = lapack_dpotri(qxx);
	qxx.make_upper_symmetric();
	return info;

#else

	cout << "ERROR: Compiled without LAPACK.\n";
	return -999;

#endif // GENLIB2_HAVE_LAPACK
}

} // namespace DEP

