/* $Id: adjintnat.h,v 1.2.2.1.2.1 2005/12/12 12:27:29 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef DEP_ADJINTNAT_INCLUDED
#define DEP_ADJINTNAT_INCLUDED

#include "adjint.h"

namespace DEP
{

/** Adjustment interface class, using native functions.
 *  Native routines are built-in functions.
 *  @author Dirk Stallmann
 */
class AdjInterfaceNative : public AdjInterface
{
public:
	/**@name solve normal equation system.
	 * Two step procedure to solve a symmetric positive definite system
	 * of linear equations.
	 * Step 1: Cholesky decomposition: A'PA = N = C'C.
	 * Only the upper triangle of N is used. This part will not be modified.
	 * The Cholesky factor C is returned in the
	 * lower triangle of N and its diagonal elements are returned
	 * in the decomposition vector. Be carefull when using operations
	 * like transpose() and make_upper_symmetric().
	 * Step 2: solving system by backsubstitution.
	 *
	 *  @param atpa (IN/OUT) array with normal matrix N in the upper part
	 *  and decomposition matrix C in the lower part
	 *  @param atpl (IN) right side vector n
	 *  @param x (OUT) solution vector x
	 *  @param dec (OUT) decomposition vector d
	 *  @return info: info = 0, OK; info = -i, invalid i-th argument;
	 *  info = i, decomposition failed in row i-1
	 */
	int solve(Matrix<double>& atpa, Vector<double>& atpl,
		Vector<double>& x, Vector<double>& dec);

	/**@name inversion of normal matrix.
	 * Compute the inverse of a symmetric positive definite matrix,
	 * using the Cholesky factorisation computed in solve.
	 *
	 *  @param atpa (IN) array with matrix C in the lower part
	 *  @param qxx (OUT) inverse of normal matrix N
	 *  @param dec (IN) decomposition vector d
	 *  @return info: info = 0, OK; info = -i, invalid i-th argument
	 */
	int inversion(Matrix<double>& atpa, Matrix<double>& qxx,
		Vector<double>& dec);
};

} // namespace DEP

#endif
