/* $Id: aolist.h,v 1.3.4.1 2005/12/12 12:27:29 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef DEP_AOLIST_INCLUDED
#define DEP_AOLIST_INCLUDED

#include "depdef.h"
#include "upari.h"

using namespace std;
using namespace GenLib2;

namespace DEP
{

class AdjObject;
class LSParams;

/** Adjustment object list class.
 *  @author Dirk Stallmann
 */
class AdjObjectList
{
public:
	/// constructor
	AdjObjectList();

	/// virtual destructor
	virtual ~AdjObjectList() {}

	/** setup object list:
	 *  (1) set upari-info and
	 *  (2) add adjustment object to the object list.
	 */
	void setup(AdjObject* obj);

	/// print list
	void print(ostream& os) const;

	/// number of adjustment objects
	int size() const { return _list.size(); }

	/// get number of unknowns
	int num_unk() const { return _num_unk; }

	/// count number of constraints
	size_t num_con() const;

	/// count number of unknown parameters by state
	int num(u_state state) const;

	/**@name count total number of ... */
	//@{
		/// - object point parameters
		int num_obptc() const;

		/// - EO parameters
		int num_eop() const;

		/// - position correction parameters
		int num_pos_cor() const;

		/// - attitude correction parameters
		int num_att_cor() const;

		/// - APs
		int num_aps() const;
	//@}

	/**@name count number of ... */
	//@{
		/// - object point parameters
		int num_obptc(u_state state) const;

		/// - EO parameters
		int num_eop(u_state state) const;

		/// - position correction parameters
		int num_pos_cor(u_state state) const;

		/// - attitude correction parameters
		int num_att_cor(u_state state) const;

		/// - APs
		int num_aps(u_state state) const;
	//@}

	/// add weighted constraints
	void add_constraints(Matrix<double>& atpa,
		Vector<double>& atpl, double& ltpl) const;

	/// stochastic weighting of the unknowns to the normal matrix
	void stoch_weighting(Matrix<double>& atpa,
		Vector<double>& atpl, double& ltpl) const;

	/// average absolute change of object coords.
	double average_change_obj(const Vector<double>& dx) const;

	/// average absolute change of projection centre coords.
	double average_change_pcc(const Vector<double>& dx) const;

	/// average absolute change of rotation parameters
	double average_change_rot(const Vector<double>& dx) const;

	/// average absolute change of image geometry parameters
	double average_change_geo(const Vector<double>& dx) const;

	/// check on convergence
	bool check_convergence(const Vector<double>& dx) const;

	/// update unknowns
	int update_unknowns(const Vector<double>& dx);

	/// update std.dev.
	int update_sdev(double sigma0, const Matrix<double>& qxx);

	/// print table of correlation coeffs.
	void print_correlations(ostream& os, const Matrix<double>& qxx,
		double ccoef_lim) const;

private:
	/// list of adjustment objects 
	vector<AdjObject*>	_list;

	/// count number of unknowns
	int		_num_unk;
};

} // namespace DEP

#endif
