/* $Id: attcor.h,v 1.4.4.1 2005/12/12 12:27:29 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef DEP_ATTCOR_INCLUDED
#define DEP_ATTCOR_INCLUDED

#include "adjobj.h"

using namespace std;
using namespace GenLib2;

namespace DEP
{

class ImageGeometry;
class DerivSub;

/** Attitude correction class.
 *  @author Dirk Stallmann
 */
class AttCorrection : public AdjObject
{
public:
	/// constructor
	AttCorrection();

	/// read attitude correction
	int read(istream& is);

	// write attitude correction
	//void write(ostream& is, SDevLim& sdev_lim);

	/// print attitude correction
	void print(ostream& os) const;

	/**@name overridden base class members */
	//@{
		/// get type name
		string type_str() const;

		/// get key name
		string key_str() const;

		/// get parameter name
		string uname(size_t i) const;

		/// set unknown parameter information (upari)
		void set_upari();

		// print unknown parameters
		//void print_unknowns(ostream& os) const;

		// print changes of unknowns
		//void print_changes(ostream& os, const Vector<double>& dx) const;
	//@}

	/// set and get reference time
	//@{
		/// set reference time
		void set_reference_time(double t) { _t0 = t; }

		/// get reference time
		double get_reference_time() const { return _t0; }
	//@}

	/// get position correction
	DPoint3 calc_corr(double t) const;
	
	/// attach EO
	int attach(vector<ImageGeometry*>& eos);

	/// get derivatives
	void get_derivs(double t, const DerivSub& deriv_eo,
		DerivSub& deriv_ac) const;

private:
	/// EOs that use this set
	vector<string>	_eo_ids;

	/// polynomial coeffs.
	vector<DPoint3>	_a;

	/// std. devs. polynom coeffs.
	vector<DPoint3>	_sa;

	/// reference time
	double			_t0;
};

} // namespace DEP

#endif
