/* $Id: ccdcam.h,v 1.3.4.1 2005/12/12 12:27:29 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef DEP_CCDCAM_INCLUDED
#define DEP_CCDCAM_INCLUDED

#include "camera.h"

using namespace std;
using namespace GenLib2;

namespace DEP
{

/** CCD frame camera class.
 *  Special class for CCD frame cameras.
 *  Additionaly to the base class this class contains parameters
 *  of the CCD sensor:
 *  \begin{itemize}
 *  \item number of sensor elements (pixel)
 *  \item pixel spacing
 *  \item location of the Image Coordinate System (ICS)
 *  \end{itemize}
 *  The location of the ICS (in pixel coordinates)
 *  is defined in the centre of the CCD array.
 *
 *	@author Dirk Stallmann
 */
class CameraCCD : public Camera
{
public:
	/// constructor
	CameraCCD();
	
	/// read
	int read(istream& is);

	/// write
	void write(ostream& os) const;

	/// print
	void print(ostream& os) const;

	/// get camera type 
	virtual string type_str() const;

	/// get (external) coordinate system type
	virtual string csys_str() const;

	/**@name external/image coordinates transformation
	 *  To convert between external image (pixel) and
	 *  (internal) image coordinates.
	 */
	//@{
	 	/// external-to-image coords. transformation
		virtual void external_to_image(const DPoint2& ext_c,
			DPoint2& img_c) const;

		/// image-to-external-coords. transformation
		virtual void image_to_external(const DPoint2& img_c,
			DPoint2& ext_c) const;
	//@}
	
	/** transformation of standard deviations
	 *  from external to image coordinates
	 */
	virtual void sdev_external_to_image(const DPoint2& ext_s,
		DPoint2& img_s) const;

	/**@name read and print image coordinates.
	 */
	//@{
		// read image coords.
		//virtual bool read_image_coords(istream& is, Image& image) const;

		/// print image coords.
		virtual void print_image_coords(ostream& os, const Image& image) const;
	//@}

protected:
	/// number of sensor elements in column direction
	int			_nc;

	/// number of sensor elements in row direction
	int			_nr;

	/// pixel spacing in column direction
	double		_sc;

	/// pixel spacing in row direction
	double		_sr;

	/// location of the ICS in column direction
	double		_c0;

	/// location of the ICS in row direction
	double		_r0;
};


// non-member functions ...

} // namespace DEP

#endif
