/* $Id: depvec.h,v 1.5.2.1.2.1 2005/12/12 12:27:29 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef DEPVEC_INCLUDED
#define DEPVEC_INCLUDED

#include <cassert>
#include <vector>
#include <string>

using namespace std;

namespace DEP
{

/** DEP vector functions.
 *  Function templates for vector class to count vector elements,
 *  find vector elements by ID number, append and print vector.
 *  @name DEP vector functions
 *  @author Dirk Stallmann (ds)
 */
//@{

/** count the number of none-NULL elements in the vector
 *  to find the number of images, cameras, etc.
 */
template<class T>
int count(const vector<T>& vec)
{
	int n = 0;

	typename vector<T>::const_iterator p;
	for (p = vec.begin(); p != vec.end(); ++p)
		{
		if (*p != 0) n++;
		}

	return n;
}


/// find object with ID in vector, return pointer to object
template<class T>
T find(const vector<T>& objects, const string& id)
{
	typename vector<T>::const_iterator p;
	for (p = objects.begin(); p != objects.end(); ++p)
		{
		if (*p != 0 && (*p)->get_id() == id)
			return *p;
		}

	return 0;
}


/** append or replace object in vector when ID exist
 *  return value: true = append, false = replace
 */
template<class T>
bool append(vector<T>& objects, T o)
{
	assert(o != 0);

	typename vector<T>::iterator p;
	for (p = objects.begin(); p != objects.end(); ++p)
		{
		if (*p != NULL && (*p)->get_id() == o->get_id())
			{
			delete (*p);
			*p = o;
			return false;
			}
		}
	
	objects.push_back(o);
	return true;
}

/** check the number of none-NULL elements in the vector
 */
template<class T>
bool is_empty(const vector<T>& vec)
{
	int n = count(vec);

	return (n == 0) ? true : false;
}

//@}

} // namespace DEP

#endif
