/* $Id: dlt.h,v 1.3.4.1 2005/12/12 12:27:29 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef DEP_DLT_INCLUDED
#define DEP_DLT_INCLUDED

#include "imggeo.h"

using namespace std;
using namespace GenLib2;

namespace DEP
{

/** Direct Linear Transformation class.
 *  The Direct Linear Transformation (DLT) is an alternate formulation
 *  of the analytical orientation problem. 
 *  Rigorous non-linear solution for the 11 unknown parameters L1, ... L11.
 *  @author Dirk Stallmann
 */
class DirectLinearTransf : public ImageGeometry
{
public:
	/// constructor
	DirectLinearTransf() : ImageGeometry(11) {}

	/**@name overridden base class AdjObject members */
	//@{
		/// get type name
		string type_str() const;

		/// get key name
		virtual string key_str() const;
	
		//// get parameter name
		string uname(size_t i) const;
	//@}

	/**@name overwrite pure virtual Image Geometry class members */
	//@{
		/// object-to-photo coords. transformation
		bool object_to_photo(const DPoint3& obj_c, DPoint3& pho_c) const;

		/// spatial intersection
		bool intersection(const ImageGeometry& g2,
			const DPoint3& pho_c1, const DPoint3& pho_c2,
			DPoint3& obj_c, double& e) const;

		/// get partial derivatives, pho_c are unused!
		bool get_derivs(const DPoint3& pho_c, const DPoint3& obj_c,
			DerivSub& deriv_oc, DerivSub& deriv_eo,
			DerivSub& deriv_pc, DerivSub& deriv_ac, DerivSub& deriv_obs);
	//@}

	/// spatial intersection
	bool intersection(const DirectLinearTransf& dlt2,
		const DPoint3& pho_c1, const DPoint3& pho_c2,
		DPoint3& obj_c, double& e) const;
};


// non-member functions ...

/// project object point onto image plane using DLT
void dlt_projection(const double* l, const DPoint3& obj_c, DPoint2& img_c);

/// intersection using DLT
bool dlt_intersection(const double* l1, const DPoint2& pho_c1, 
const double* l2, const DPoint2& pho_c2, DPoint3& obj_c);

} // namespace DEP

#endif
