/* $Id: eoaustralis.cpp,v 1.3.4.1 2005/12/12 12:27:49 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "eoaustralis.h"

namespace DEP
{

// get type name

string EOAustralis::type_str() const
{
	ostringstream oss;
	oss << "EO (Australis)";
	return oss.str();
}


// get key name

string EOAustralis::key_str() const
{
	ostringstream oss;
	oss << "ext-ori-australis";
	return oss.str();
}


// get parameter name

string EOAustralis::uname(size_t i) const
{
	ostringstream oss;
	
	switch (i)
		{
		case 0: oss << "Xo"; break;
		case 1: oss << "Yo"; break;
		case 2: oss << "Zo"; break;
		case 3: oss << "az"; break;
		case 4: oss << "elev"; break;
		case 5: oss << "roll"; break;
		default:
			assert(false);
			break;
		}

	return oss.str();
}


// get partial derivatives of rotation parameters

void EOAustralis::get_derivs_rot(DerivSub& deriv_eo) const
{
	//double sin_a = sin(_rot_angles_cor.x);
	//double cos_a = cos(_rot_angles_cor.x);
	//double sin_e = sin(_rot_angles_cor.y);
	//double cos_e = cos(_rot_angles_cor.y);
	double sin_r = sin(_rot_params[2]);
	double cos_r = cos(_rot_params[2]);

	// azimuth-derivatives
	deriv_eo(0,3) = _cn2 * (_n * (-_rot_matrix(1,0) * _dx
		+ _rot_matrix(0,0) * _dy)
		- _zx * (-_rot_matrix(1,2) * _dx + _rot_matrix(0,2) * _dy));
	deriv_eo(1,3) = _cn2 * (_n * (-_rot_matrix(1,1) * _dx
		+ _rot_matrix(0,1) * _dy)
		- _zy * (-_rot_matrix(1,2) * _dx + _rot_matrix(0,2) * _dy));

	// elevation-derivatives
	// this is equivalent:
	//deriv_eo(0,4) = cn2 * (n * n * sin_r - zx *
	//	(-sin_a * sin_e * dx + cos_a * sin_e * dy - cos_e * dz));
	//deriv_eo(1,4) = cn2 * (n * n * cos_r - zy *
	//	(-sin_a * sin_e * dx + cos_a * sin_e * dy - cos_e * dz));
	// to this:
	deriv_eo(0,4) = _cn2 * (_n * _n * sin_r
		+ _zx * (sin_r * _zx + cos_r * _zy));
	deriv_eo(1,4) = _cn2 * (_n * _n * cos_r
		+ _zy * (sin_r * _zx + cos_r * _zy));

	// roll-derivatives
	deriv_eo(0,5) =  _cn * _zy;
	deriv_eo(1,5) = -_cn * _zx;
}


// non-member functions ...

} // namespace DEP
