/* $Id: eoopkfix.cpp,v 1.4.4.1 2005/12/12 12:27:49 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "eoopkfix.h"

namespace DEP
{

// get type name

string EOAnglesOPKFix::type_str() const
{
	ostringstream oss;
	oss << "EO (omega-phi-kappa, fixed)";
	return oss.str();
}


// get key name

string EOAnglesOPKFix::key_str() const
{
	ostringstream oss;
	oss << "ext-ori-opk-fix";
	return oss.str();
}


// get parameter name

string EOAnglesOPKFix::uname(size_t i) const
{
	ostringstream oss;
	
	switch (i)
		{
		case 0: oss << "Xo"; break;
		case 1: oss << "Yo"; break;
		case 2: oss << "Zo"; break;
		case 3: oss << "omega"; break;
		case 4: oss << "phi"; break;
		case 5: oss << "kappa"; break;
		default:
			assert(false);
			break;
		}

	return oss.str();
}


// get partial derivatives of rotation parameters

void EOAnglesOPKFix::get_derivs_rot(DerivSub& deriv_eo) const
{
	double sin_o = sin(_rot_params[0]);
	double cos_o = cos(_rot_params[0]);
	double sin_p = sin(_rot_params[1]);
	double cos_p = cos(_rot_params[1]);
	double sin_k = sin(_rot_params[2]);
	double cos_k = cos(_rot_params[2]);

	// omega-derivatives
	deriv_eo(0,3) = _cn2 * (- _zx * _zy);
	deriv_eo(1,3) = _cn2 * (-(_n * _n) - (_zy * _zy));

	// phi-derivatives
	deriv_eo(0,4) = _cn2 * (_n * ((-cos_k * _dx + sin_k * _dy) * sin_p
		+ cos_p * _dz)
		- _zx * (((-cos_k * _dx + sin_k * _dy) * cos_p - sin_p * _dz) * cos_o));
	deriv_eo(1,4) = _cn2 * (_n * (((cos_k * _dx - sin_k * _dy) * cos_p
		+ sin_p * _dz) * sin_o)
		- _zy * (((-cos_k * _dx + sin_k * _dy) * cos_p - sin_p * _dz) * cos_o));

	// kappa-derivatives
	deriv_eo(0,5) = _cn2 * (_n * (_rot_matrix(1,0) * _dx
		- _rot_matrix(0,0) * _dy)
		- _zx * (_rot_matrix(1,2) * _dx - _rot_matrix(0,2) * _dy));
	deriv_eo(1,5) = _cn2 * (_n * (_rot_matrix(1,1) * _dx
		- _rot_matrix(0,1) * _dy)
		- _zy * (_rot_matrix(1,2) * _dx - _rot_matrix(0,2) * _dy));
}


// non-member functions ...

} // namespace DEP
