/* $Id: eoopkfix.h,v 1.5.4.1 2005/12/12 12:27:49 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef DEP_EOOPKFIX_INCLUDED
#define DEP_EOOPKFIX_INCLUDED

#include "eopokrot.h"

using namespace std;
using namespace GenLib2;

namespace DEP
{

/** Exterior orientation using euler angles.
 *   
 *    Parameters            : 3 rotation angles (euler angles)
 *    Coordinate axes       : fixed
 *    Sequence of rotations : Rz(kappa) * Ry(phi) * Rx(omega)
 *    Direction of rotation : counter-clockwise
 *
 *  @author Dirk Stallmann
 */
class EOAnglesOPKFix : public EOAnglesPOKRot
{
public:
	/// constructor
	EOAnglesOPKFix() : EOAnglesPOKRot() {}

	/**@name overridden base class members of AdjObject */
	//@{
		/// get type name
		string type_str() const;

		/// get key name
		string key_str() const;
	
		//// get parameter name
		string uname(size_t i) const;
	//@}

	/**@name overridden class members of EOAnglesPOKRot */
	//@{
		/// calculate rotation parameters from rotation matrix
		virtual void calc_rot_params(vector<double>& a,
			const RotMatrix3<double>& R) const {
			R.get_angles_opk_fix(a[0], a[1], a[2]);
		}

		/// calculate rotation matrix from rotation parameters
		virtual void calc_rot_matrix(RotMatrix3<double>& R,
			const vector<double>& a) const {
			R.set_angles_opk_fix(a[0], a[1], a[2]);
		}
	
		/// get partial derivatives of rotation parameters
		void get_derivs_rot(DerivSub& deriv_eo) const;

	//@}
};


// non-member functions ...

} // namespace DEP

#endif
