/* $Id: eoopkrot.cpp,v 1.3.4.1 2005/12/12 12:27:49 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "eoopkrot.h"

namespace DEP
{

// get type name

string EOAnglesOPKRot::type_str() const
{
	ostringstream oss;
	oss << "EO (omega-phi-kappa, rotated)";
	return oss.str();
}


// get key name

string EOAnglesOPKRot::key_str() const
{
	ostringstream oss;
	oss << "ext-ori-opk-rot";
	return oss.str();
}


// get parameter name

string EOAnglesOPKRot::uname(size_t i) const
{
	ostringstream oss;
	
	switch (i)
		{
		case 0: oss << "Xo"; break;
		case 1: oss << "Yo"; break;
		case 2: oss << "Zo"; break;
		case 3: oss << "omega"; break;
		case 4: oss << "phi"; break;
		case 5: oss << "kappa"; break;
		default:
			assert(false);
			break;
		}

	return oss.str();
}


// get partial derivatives of rotation parameters

void EOAnglesOPKRot::get_derivs_rot(DerivSub& deriv_eo) const
{
	// omega-derivatives
	deriv_eo(0,3) = _cn * ((_dy * _rot_matrix(2,2) - _dz * _rot_matrix(1,2))
		* _zx / _n - _dy * _rot_matrix(2,0) + _dz * _rot_matrix(1,0));
	deriv_eo(1,3) = _cn * ((_dy * _rot_matrix(2,2) - _dz * _rot_matrix(1,2))
		* _zy / _n - _dy * _rot_matrix(2,1) + _dz * _rot_matrix(1,1));

	double sin_k = sin(_rot_params[2]);
	double cos_k = cos(_rot_params[2]);

	// phi-derivatives
	deriv_eo(0,4) = -_cn * ((_zx * cos_k - _zy * sin_k) * _zx / _n
		+ _n * cos_k);
	deriv_eo(1,4) = -_cn * ((_zx * cos_k - _zy * sin_k) * _zy / _n
		- _n * sin_k);

	// kappa-derivatives
	deriv_eo(0,5) =  _cn * _zy;
	deriv_eo(1,5) = -_cn * _zx;
}


// non-member functions ...

} // namespace DEP
