/* $Id: eopokrot.h,v 1.5.4.1 2005/12/12 12:27:49 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef DEP_EOPOKROT_INCLUDED
#define DEP_EOPOKROT_INCLUDED

#include "extori3.h"

using namespace std;
using namespace GenLib2;

namespace DEP
{

/** Exterior orientation using euler angles class.
 *   
 *    Parameters            : 3 rotation angles (euler angles)
 *    Coordinate axes       : rotated
 *    Sequence of rotations : Ry(phi) * Rx(omega) * Rz(kappa)
 *    Direction of rotation : counter-clockwise
 *
 *  @author Dirk Stallmann
 */
class EOAnglesPOKRot : public ExtOrientation3
{
public:
	/// constructor
	EOAnglesPOKRot() : ExtOrientation3() {}

	/**@name overridden base class members of AdjObject */
	//@{
		/// get type name
		virtual string type_str() const;

		/// get key name
		virtual string key_str() const;
	
		/// get parameter name
		virtual string uname(size_t i) const;

		/// sum of absolute changes
		//void sum_changes(const Vector<double>& dx,
		//	double& sum, size_t& num) const {}
	//@}

	/**@name overridden class members of ExtOrientation */
	//@{
		/// read EO parameters
		int read(istream& is);

		/// write EO parameters
		void write(ostream& os) const;

		/// print EO parameters
		void print(ostream& os) const;

		/// print changes of unknowns
		void print_changes(ostream& os, const Vector<double>& dx) const;

		/// print unknown parameters
		void print_unknowns(ostream& os) const;

		/// calculate rotation parameters from rotation matrix
		virtual void calc_rot_params(vector<double>& a,
			const RotMatrix3<double>& R) const {
			R.get_angles_pok_rot(a[0], a[1], a[2]);
		}

		/// calculate rotation matrix from rotation parameters
		virtual void calc_rot_matrix(RotMatrix3<double>& R,
			const vector<double>& a) const {
			R.set_angles_pok_rot(a[0], a[1], a[2]);
		}
		
		/// get partial derivatives of rotation parameters
		virtual void get_derivs_rot(DerivSub& deriv_eo) const;
	//@}

	/// set rotation angles
	//@{
		/// set rotation angles
		void set_rot_angles(const DPoint3& rot) {
			_a[3] = rot.x; _a[4] = rot.y; _a[5] = rot.z;
		}

		/// set std. devs. of rotation angles
		void set_sdev_rot(const DPoint3& sdev) {
			_sa[3] = sdev.x; _sa[4] = sdev.y; _sa[5] = sdev.z;
		}
	//@}
};


// non-member functions ...

} // namespace DEP

#endif
