/* $Id: eoquaternion.h,v 1.3.4.1 2005/12/12 12:27:49 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef DEP_EOQUATERNION_INCLUDED
#define DEP_EOQUATERNION_INCLUDED

#include "extori.h"

using namespace std;
using namespace GenLib2;

namespace DEP
{

/** Exterior orientation using quaternion class.
 *  Imaging geometry
 *  Parameters: 3 translations, 4 rotations
 *  Constraints: 1 between the rotation parameters
 *   
 *  @author Dirk Stallmann
 */
class EOQuaternion : public ExtOrientation
{
public:
	/// constructor
	EOQuaternion() : ExtOrientation(4) {}

	/**@name overridden base class members of AdjObject */
	//@{
		/// get type name
		string type_str() const;

		/// get key name
		string key_str() const;
	
		//// get parameter name
		string uname(size_t i) const;

		/// get number of constraints
		size_t num_con() const { return 1; }

		/// add weighted constraints
		void add_constraints(Matrix<double>& atpa,
			Vector<double>& atpl, double& ltpl) const;
	//@}

	/**@name overridden class members of ExtOrientation */
	//@{
		/// get partial derivatives of rotation parameters
		void get_derivs_rot(DerivSub& deriv_eo) const;

		/// calculate rotation parameters from rotation matrix
		virtual void calc_rot_params(vector<double>& a,
			const RotMatrix3<double>& R) const {
			R.get_quaternion(a[0], a[1], a[2], a[3]);
		}

		/// calculate rotation matrix from rotation parameters
		virtual void calc_rot_matrix(RotMatrix3<double>& R,
			const vector<double>& a) const {
			R.set_quaternion(a[0], a[1], a[2], a[3]);
		}
	//@}

protected:
	/// differential rotation matrices
	//@{
		/// partial derivatives for q0
		void dr_q0(RotMatrix3<double>& dR) const;

		/// partial derivatives for q1
		void dr_q1(RotMatrix3<double>& dR) const;

		/// partial derivatives for q2
		void dr_q2(RotMatrix3<double>& dR) const;

		/// partial derivatives for q3
		void dr_q3(RotMatrix3<double>& dR) const;
	//@}
};


// non-member functions ...

} // namespace DEP

#endif
