/* $Id: extori3.cpp,v 1.2.4.1 2005/12/12 12:27:49 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "deperr.h"
#include "extori3.h"
#include "attcor.h"
#include "poscor.h"

namespace DEP
{

// constructor
ExtOrientation3::ExtOrientation3() : ExtOrientation(3), _att_cor(0) {}


// get partial derivatives of position and attitude correction parameters

void ExtOrientation3::get_derivs_rot_acor(DerivSub& deriv_eo,
DerivSub& deriv_ac) const
{
	// get derivatives of the rotation parameters
	get_derivs_rot(deriv_eo);

	// get derivatives of attitude correction parameters
	if (_att_cor != 0)
		_att_cor->get_derivs(_time, deriv_eo, deriv_ac);
}


// calculate EO, position and attitude correction included

void ExtOrientation3::calc_orientation()
{
	// perspective centre coords.
	_per_centre.x = _a[0];
	_per_centre.y = _a[1];
	_per_centre.z = _a[2];

	// get position correction (optional)
	if (_pos_cor != 0)
		_per_centre += _pos_cor->calc_corr(_time);

	// rotation parameters
	for (size_t i = 3, j = 0; i < _size; i++, j++)
		_rot_params[j] = _a[i];

	// get attitude correction (optional)
	if (_att_cor != 0)
		{
		DPoint3 att_cor = _att_cor->calc_corr(_time);

		_rot_params[0] += att_cor.x;
		_rot_params[1] += att_cor.y;
		_rot_params[2] += att_cor.z;
		}
	
	// calculate rotation matrix
	calc_rot_matrix(_rot_matrix, _rot_params);
}


// non-member functions ...

} // namespace DEP
