/* $Id: extori3.h,v 1.3.4.1 2005/12/12 12:27:49 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef DEP_EXTORI3_INCLUDED
#define DEP_EXTORI3_INCLUDED

#include "extori.h"

using namespace std;
using namespace GenLib2;

namespace DEP
{
class AttCorrection;

/** Exterior orientation using 3 rotation parameters class.
 *  Intermediate class for classes using 3 rotation parameters,
 *  e.g. Euler angles, Rodrigues parameter.
 *  The attitude correction can simply only applied for 3 rotation
 *  parameters.
 *
 *  @author Dirk Stallmann
 */
class ExtOrientation3 : public ExtOrientation
{
public:
	/// constructor
	ExtOrientation3();
	
	/// attach attitude correction
	void attach(AttCorrection* p) { _att_cor = p; }

	/// check link to attitude correction
	bool has_att_cor() const { return _att_cor != 0; }

	/// get partial derivatives of rotation and attitude correction parameters
	void get_derivs_rot_acor(DerivSub& deriv_eo,
		DerivSub& deriv_ac) const;
	
	/// get partial derivatives of rotation parameters
	virtual void get_derivs_rot(DerivSub& deriv_eo) const = 0;

	/// calculate EO, position correction included
	void calc_orientation();

protected:
	/// attitude correction (optional)
	AttCorrection*		_att_cor;
};


// non-member functions ...

} // namespace DEP

#endif
