/* $Id: fiodep.cpp,v 1.9.2.1.2.2 2007/03/06 08:49:43 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "deperr.h"
#include "depvec.h"
#include "fiodep.h"
#include "lspars.h"
#include "image.h"
//#include "scanner.h"
#include "imgpnt.h"
#include "objpnt.h"
#include "ccdcamtr.h"
#include "pbcam.h"
#include "eopokrot.h"
#include "eoopkrot.h"
#include "eoopkfix.h"
#include "eoaustralis.h"
#include "eoquaternion.h"
#include "dlt.h"
//#include "eokratky.h"
#include "pmflin.h"
#include "apset.h"
#include "poscor.h"
#include "attcor.h"

static char* tbl[] = {		// keywords
	"placeholder",

	"params",
	"ls-params",

	"camera",
		"camera-frame",
		"camera-ccd",
		"camera-ccd-tr",
		"camera-pushbroom",

	"image",
		"image-frame",
		"image-scanner",

	"image-geometry",
	"ext-ori",
		"ext-ori-opk-rot",
		"ext-ori-pok-rot",
		"ext-ori-opk-fix",
		"ext-ori-australis",
		"ext-ori-quaternion",
		"dlt",
		"ext-ori-kratky",
		"pmf-linear",

	"image-crds",
	"obj-crds",
	"ctrl-supp",

	"ap-set",
		"gap",
		"australis",
		"p12",
		"p44",
		"inner-or",
		"radial-dist",
		"decentering-dist",
		"in-plane-dist",
		"out-of-plane-dist",

	"pos-cor",
	"att-cor",

	"patb-params",
	"patb-photos",
	"patb-control",
	"patb-orient",

	"oc-transform",

	"stop-dep"
};

namespace DEP
{

// test keyword

extern fio_type fio_test(const char* keyword)
{
	if (strcmp(keyword, tbl[PARAMS]) == 0) return PARAMS;
	if (strcmp(keyword, tbl[LS_PARAMS]) == 0) return LS_PARAMS;

	if (strcmp(keyword, tbl[CAMERA]) == 0) return CAMERA;
	if (strcmp(keyword, tbl[CAMERA_FRAME]) == 0) return CAMERA_FRAME;
	if (strcmp(keyword, tbl[CAMERA_CCD]) == 0) return CAMERA_CCD;
	if (strcmp(keyword, tbl[CAMERA_CCD_TR]) == 0) return CAMERA_CCD_TR;
	if (strcmp(keyword, tbl[CAMERA_PUSHBROOM]) == 0) return CAMERA_PUSHBROOM;

	if (strcmp(keyword, tbl[IMAGE]) == 0) return IMAGE;
	if (strcmp(keyword, tbl[IMAGE_FRAME]) == 0) return IMAGE_FRAME;
	if (strcmp(keyword, tbl[IMAGE_SCANNER]) == 0) return IMAGE_SCANNER;

	if (strcmp(keyword, tbl[IMAGE_GEOMETRY]) == 0) return IMAGE_GEOMETRY;
	if (strcmp(keyword, tbl[EXT_ORI]) == 0) return EXT_ORI;
	if (strcmp(keyword, tbl[EXT_ORI_OPK_ROT]) == 0) return EXT_ORI_OPK_ROT;
	if (strcmp(keyword, tbl[EXT_ORI_POK_ROT]) == 0) return EXT_ORI_POK_ROT;
	if (strcmp(keyword, tbl[EXT_ORI_OPK_FIX]) == 0) return EXT_ORI_OPK_FIX;
	if (strcmp(keyword, tbl[EXT_ORI_AUSTRALIS]) == 0) return EXT_ORI_AUSTRALIS;
	if (strcmp(keyword, tbl[EXT_ORI_QUATERNION]) == 0)
		return EXT_ORI_QUATERNION;
	if (strcmp(keyword, tbl[DLT]) == 0) return DLT;
	if (strcmp(keyword, tbl[EXT_ORI_KRATKY]) == 0) return EXT_ORI_KRATKY;
	if (strcmp(keyword, tbl[PMF_LINEAR]) == 0) return PMF_LINEAR;

	if (strcmp(keyword, tbl[IMAGE_COORDS]) == 0) return IMAGE_COORDS;
	if (strcmp(keyword, tbl[OBJECT_COORDS]) == 0) return OBJECT_COORDS;
	if (strcmp(keyword, tbl[CONTROL_SUPP]) == 0) return CONTROL_SUPP;

	if (strcmp(keyword, tbl[APSET]) == 0) return APSET;
	if (strcmp(keyword, tbl[APSET_GAP]) == 0) return APSET_GAP;
	if (strcmp(keyword, tbl[APSET_AUSTRALIS]) == 0) return APSET_AUSTRALIS;
	if (strcmp(keyword, tbl[APSET_P12]) == 0) return APSET_P12;
	if (strcmp(keyword, tbl[APSET_P44]) == 0) return APSET_P44;
	if (strcmp(keyword, tbl[APSET_INNER_OR]) == 0) return APSET_INNER_OR;
	if (strcmp(keyword, tbl[APSET_RADIAL]) == 0) return APSET_RADIAL;
	if (strcmp(keyword, tbl[APSET_DECENTERING]) == 0) return APSET_DECENTERING;
	if (strcmp(keyword, tbl[APSET_INPLANE]) == 0) return APSET_INPLANE;
	if (strcmp(keyword, tbl[APSET_OUTOFPLANE]) == 0) return APSET_OUTOFPLANE;

	if (strcmp(keyword, tbl[POS_COR]) == 0) return POS_COR;
	if (strcmp(keyword, tbl[ATT_COR]) == 0) return ATT_COR;

	if (strcmp(keyword, tbl[PATB_PARAMS]) == 0) return PATB_PARAMS;
	if (strcmp(keyword, tbl[PATB_PHOTOGRAPHS]) == 0) return PATB_PHOTOGRAPHS;
	if (strcmp(keyword, tbl[PATB_CONTROL_POINTS]) == 0)
		return PATB_CONTROL_POINTS;
	if (strcmp(keyword, tbl[PATB_ORIENT_PARAMS]) == 0)
		return PATB_ORIENT_PARAMS;

	if (strcmp(keyword, tbl[OC_TRANSFORM]) == 0)
		return OC_TRANSFORM;

	if (strcmp(keyword, tbl[STOP_DEP]) == 0)
		return STOP_DEP;

	return UNKNOWN;
}


// read cameras

int read_cameras(istream& is, vector<Camera*>& cams)
{
	string str;

	while (is >> str)
		{
		fio_type type = fio_test(str.c_str());
		if (type == STOP_DEP) break;

		// create new camera
		Camera* cam = 0;
		switch (type)
			{
			case CAMERA_FRAME:	cam = new Camera; break;
			case CAMERA_CCD:	cam = new CameraCCD; break;
			case CAMERA_CCD_TR:	cam = new CameraCCDTR; break;
			case CAMERA_PUSHBROOM:	cam = new PBCamera; break;

			case UNKNOWN:
			default:
				cout << "ERROR: Unknown camera type: " << str << endl;
				return ERROR;
			}
		assert(cam != 0);

		// read camera data
		if (cam->read(is) != OK)
			{
			cout << "ERROR: Reading camera " << cam->get_id() << " failed.\n";	
			delete cam;
			return ERROR;
			}

		// append or replace camera
		if (append(cams, cam) == false)
			cout << "ATTENTION: Camera " << cam->get_id() << " was replaced.\n";
		}

	return OK;
}


// read images

int read_images(istream& is, const vector<Camera*>& cams,
vector<Image*>& images)
{
	string str;

	while (is >> str)
		{
		fio_type type = fio_test(str.c_str());
		if (type == STOP_DEP) break;

		// create new image
		Image* image = 0;
		switch (type)
			{
			case IMAGE_FRAME: image = new Image; break; 
			// case IMAGE_SCANNER: image = new Scanner; break; 

			case UNKNOWN:
			default:
				cout << "ERROR: Unknown image type: " << str << endl;
				return ERROR;
			}
		//image = new Image;
		assert(image != 0);
		
		// read image information
		if (image->read(is, cams) != OK)
			{
			cout << "ERROR: Reading image " << image->get_id() << " failed.\n";
			delete image;
			return ERROR;
			}
			
		// append or replace
		if (append(images, image) == false)
			cout << "ATTENTION: Image " << image->get_id()
				<< " was replaced.\n";
		}

	return OK;
}


// read exterior orientations

int read_ext_orientation(istream& is, vector<ImageGeometry*>& eos)
{
	string str;

	if (!(is >> str))
		{
		cout << "ERROR: Reading keyword failed.\n";
		return ERROR;
		}

	if (str.compare("indiv-type") == 0)
		{
		if (read_ext_orientation_indiv(is, eos) != OK)
			{
			cout << "ERROR: Reading EO individual type failed.\n";
			return ERROR;
			}
		}
	else if (str.compare("common-type") == 0)
		{
		if (read_ext_orientation_common(is, eos) != OK)
			{
			cout << "ERROR: Reading EO common type failed.\n";
			return ERROR;
			}
		}
	else
		{
		cout << "ERROR: Unknown keyword: " << str << endl;
		return ERROR;
		}

	return OK;
}


// read EO have individual type and std. devs.

int read_ext_orientation_indiv(istream& is, vector<ImageGeometry*>& eos)
{
	string str;

	while (is >> str)
		{
		fio_type type = fio_test(str.c_str());
		if (type == STOP_DEP) break;

		// create new image geometry
		ImageGeometry* eo = 0;
		switch (type)
			{
			case EXT_ORI_OPK_ROT: eo = new EOAnglesOPKRot; break;
			case EXT_ORI_POK_ROT: eo = new EOAnglesPOKRot; break;
			case EXT_ORI_OPK_FIX: eo = new EOAnglesOPKFix; break;
			case EXT_ORI_AUSTRALIS: eo = new EOAustralis; break;
			case EXT_ORI_QUATERNION: eo = new EOQuaternion; break;
			case DLT: eo = new DirectLinearTransf; break;
			//case EXT_ORI_KRATKY: eo = new EOKratky; break;
			case PMF_LINEAR: eo = new PMFLinear; break;

			case UNKNOWN:
			default:
				cout << "ERROR: Unknown type of image geometry: "
					<< str << endl;	
				return ERROR;
			}
		assert(eo != 0);

		// read EO data
		if (eo->read(is) != OK)
			{
			cout << "ERROR: Reading exterior orientation failed.\n";
			delete eo;
			return ERROR;
			}

		// append or replace
		if (append(eos, eo) == false)
			cout << "ATTENTION: EO " << eo->get_id() << " was replaced.\n";
		}

	return OK;
}


// read EO have common type and std. devs.

int read_ext_orientation_common(istream& is, vector<ImageGeometry*>& eos)
{
	string str;

	// read type of geometry
	is >> str;
	if (!is)
		{
		cout << "ERROR: Cannot read type of geometry.\n";
		return ERROR;
		}

	// create new image geometry
	fio_type type = fio_test(str.c_str());
	
	// create dummy object
	ExtOrientation* eo = 0;
	switch (type)
		{
		case EXT_ORI_OPK_ROT: eo = new EOAnglesOPKRot; break;
		case EXT_ORI_POK_ROT: eo = new EOAnglesPOKRot; break;
		case EXT_ORI_OPK_FIX: eo = new EOAnglesOPKFix; break;
		case EXT_ORI_AUSTRALIS: eo = new EOAustralis; break;
		case EXT_ORI_QUATERNION: eo = new EOQuaternion; break;
		//case DLT: eo = new DirectLinearTransf; break;

		case UNKNOWN:
		default:
			cout << "ERROR: Unknown type of image geometry: " << str << endl;	
			return ERROR;
		}
	assert(eo != 0);
	
	// unit of rotation angles
	is >> str;
	if (!is)
		{
		cout << "ERROR: Cannot read unit of rotation angles.\n";
		return ERROR;
		}

	UnitAngle u_angle;
	if (!u_angle.set(str.c_str()))
		{
		cout << "ERROR: Invalid unit of rotation angles: " << str << endl;
		return ERROR;
		}
	double rho = u_angle.rho();

	// time flag
	bool time_flg = true;
	if (!(is >> time_flg))
		{
		cout << "ERROR: Cannot read time flag.\n";
		return ERROR;
		}
	
	// rotation matrix flag: 
	bool matrix_flg = false;
	is >> matrix_flg;
	if (!is)
		{
		cout << "ERROR: Cannot read rotation matrix flag.\n";
		return ERROR;
		}

	// read common std. dev.
	DPoint3 sdev_pcc;
	vector<double> sa(3);

	if (!(is >> sdev_pcc >> sa[0] >> sa[1] >> sa[2]))
		{
		cout << "ERROR: Cannot read common std. devs.\n";
		return ERROR;
		}

	// only for real std. devs. conversion into radian
	if (AdjObject::lspars.is_rsdev(sa[0])) sa[0] /= rho;
	if (AdjObject::lspars.is_rsdev(sa[1])) sa[1] /= rho;
	if (AdjObject::lspars.is_rsdev(sa[2])) sa[2] /= rho;

	// number of rotation parameters
	//size_t n = eo->size() - 3;
	
	//double* a = new double[n];
	//assert(a != 0);

	//double* sa = new double[n];
	//assert(sa != 0);

	//size_t i = 0;
	//for (i = 0; i < n; i++)
	//	{
	//	if (!(is >> sa[i]))
	//		{
	//		cout << "ERROR: Cannot read common std. devs.\n";
	//		return ERROR;
	//		}
	//	}

	// only for real std. devs. conversion into radian
	//for (i = 0; i < n; i++)
	//	{
	//	if (AdjObject::lspars.is_rsdev(sa[i]))
	//		sa[i] /= rho;
	//	}

	string id;
	double time = 0.;
	DPoint3 pcc;
	vector<double> a(3);
	RotMatrix3<double> R;

	delete eo; // delete dummy

	while (is >> id)
		{
		if (id.compare("stop-dep") == 0 || id.compare("-1") == 0) break;

		// create new image geometry
		switch (type)
			{
			case EXT_ORI_OPK_ROT: eo = new EOAnglesOPKRot; break;
			case EXT_ORI_POK_ROT: eo = new EOAnglesPOKRot; break;
			case EXT_ORI_OPK_FIX: eo = new EOAnglesOPKFix; break;
			case EXT_ORI_AUSTRALIS: eo = new EOAustralis; break;
			case EXT_ORI_QUATERNION: eo = new EOQuaternion; break;
			//case DLT: eo = new DirectLinearTransf; break;

			case UNKNOWN:
			default:
				assert(false);	
			}
		assert(eo != 0);

		if (time_flg)
			{
			if (!(is >> time))
				{
				cout << "ERROR: Cannot read exposure time.\n";
				return ERROR;
				}
			}

		if (!(is >> pcc))
			{
			cout << "ERROR: Cannot read projection centre coords.\n";
			return ERROR;
			}

		//for (i = 0; i < n; i++)
		//	{
		//	if (!(is >> a[i]))
		//		{
		//		cout << "ERROR: Cannot read geometry parameters.\n";
		//		return ERROR;
		//		}
		//	}

		if (matrix_flg == false)
			{
			if (!(is >> a[0] >> a[1] >> a[2]))
				{
				cout << "ERROR: Cannot read rotation parameters.\n";
				return ERROR;
				}
				
			a[0] /= rho; // conversion into radian
			a[1] /= rho; // conversion into radian
			a[2] /= rho; // conversion into radian
			}
		else
			{
			if (!(is >> R))
				{
				cout << "ERROR: Cannot read rotation matrix.\n";
				return ERROR;
				}
				
			eo->calc_rot_params(a, R); // get rotation angles
			}

		// set member data
		eo->set_id(id);
		eo->set_time(time);
		//eo->set_params(a);
		//eo->set_sdevs(sa);
		eo->set_pcc(pcc);
		eo->set_sdev_pcc(sdev_pcc);
		eo->set_rot_params(a);
		eo->set_sdev_rot(sa);

		// append or replace
		if (append(eos, dynamic_cast<ImageGeometry*>(eo)) == false)
			cout << "ATTENTION: Exterior orientation " << id
				<< " was replaced.\n";
		}

	return OK;
}


// read image coordinates

int read_image_coords(istream& is, vector<Image*>& images)
{
	string id;

	while (is >> id)
		{
		if (id.compare("stop-dep") == 0 || id.compare("-1") == 0) break;

		// find image
		Image* image = find(images, id);
		if (image == 0)
			{
			cout << "ERROR: Image " << id << " not found.\n";
			return ERROR;
			}

		// append or replace image coords.
		if (image->read_image_coords(is) != OK)
			{
			cout << "ERROR: Image " << id << " read image coords. failed.\n";
			return ERROR;
			}
		}

	return OK;
}


// read object point coords.

int read_object_coords(istream& is, vector<ObjectPoint*>& obpts)
{
	DPoint3 crds, sdev;
	string id;
	string str;
	
	bool com_sdev = false;

	is >> str;
	if (!is) return ERROR;

	if (str.compare("indiv-sdev") == 0)
		com_sdev = false;
	else if (str.compare("common-sdev") == 0)
		com_sdev = true;
	else
		{
		cout << "ERROR: Unknown file type: " << str << endl;
		return ERROR;
		}

	// read common std. dev.
	if (com_sdev)
		{
		is >> sdev;
		if (!is)
			{
			cout << "ERROR: Reading common std. dev. failed.\n";
			return ERROR;
			}
		}

	while (is >> id)
		{
		if (id.compare("stop-dep") == 0 || id.compare("-1") == 0)
			break; // stop reading

		// read coords.
		is >> crds;
		if (!is)
			{
			cout << "ERROR: Reading coords. failed.\n";
			return ERROR;
			}

		// read individual std. dev.
		if (!com_sdev)
			{
			is >> sdev;
			if (!is)
				{
				cout << "ERROR: Reading std. dev. failed.\n";
				return ERROR;
				}
			}
			
		ObjectPoint* op = find(obpts, id);
		if (op != 0)
			cout << "ATTENTION: Object point " << id << " replaced.\n";
		else
			{
			// create new object point
			op = new ObjectPoint;
			assert(op != 0);

			op->set_id(id);
			obpts.push_back(op);
			}

		// store coords. and std. dev.
		op->crds = crds;
		op->sdev = sdev;
		op->approx(true);

		// set type from std. dev.
		// Attention: type and std. devs. must be correspond!!!
		op->set_type();
		}

	return OK;
}


// read control support definition

int read_ctrl_supp(istream& is, vector<ObjectPoint*>& obpts)
{
	for (;;)
		{
		ObjectPointCS cs;

		// read CS
		if (cs.read_ctrl_supp(is) != OK)
			{
			cout << "ERROR: Set std. dev. from CS failed.\n";
			return ERROR;
			}

		if (cs.id.compare("stop-dep") == 0 || cs.id.compare("-1") == 0)
			break; // stop reading

		// find object point
		ObjectPoint* op = find(obpts, cs.id);
		if (op == 0)
			{
			cout << "ERROR: Object point ID " << cs.id << " not exist.\n";
			return ERROR;
			}

		// set std. dev. from CS
		if ((op->set_sdev(cs)) != OK)
			{
			cout << "ERROR: Set std. dev. from CS failed: " << cs.id << endl;
			return ERROR;
			}
		}

	return OK;
}


// read AP sets

int read_ap_sets(istream& is, vector<APSet*>& aps)
{
	string str;

	while (is >> str)
		{
		fio_type type = fio_test(str.c_str());
		if (type == STOP_DEP) break;

		// create new AP set
		APSet* ap = 0;
		switch (type)
			{
			//case APSET_INNER_OR:	ap = new APSet; break;
			case APSET_GAP:			ap = new APSetGAP; break;
			case APSET_AUSTRALIS:	ap = new APSetAustralis; break;
			case APSET_P12:			ap = new APSetP12; break;
			case APSET_P44:			ap = new APSetP44; break;
			case APSET_INNER_OR:	ap = new APSetIO; break;
			case APSET_RADIAL:  	ap = new APSetRadial; break;
			case APSET_DECENTERING: ap = new APSetDecentering; break;
			case APSET_INPLANE: 	ap = new APSetInPlane; break;
			case APSET_OUTOFPLANE: 	ap = new APSetOutOfPlane; break;

			case UNKNOWN:
			default:
				cout << "ERROR: Read AP set: Unknown type: " << str << endl;	
				return ERROR;
			}
		assert(ap != 0);

		// read ap set
		//if (ap->read(is, lspars) != OK)
		if (ap->read(is) != OK)
			{
			cout << "ERROR: Read AP set " << ap->get_id() << " failed.\n";
			delete ap;
			return ERROR;
			}

		// append or replace
		if (append(aps, ap) == false)
			cout << "ATTENTION: AP set " << ap->get_id() << " was replaced.\n";
		}

	return OK;
}


// read position corrections

int read_pos_correction(istream& is, vector<PosCorrection*>& pcors)
{
	string id;

	while (is >> id)
		{
		if (id.compare("stop-dep") == 0) break;

		// create new object
		PosCorrection* pcor = new PosCorrection;
		assert(pcor != 0);

		// set ID
		pcor->set_id(id);

		// read data
		if (pcor->read(is) != OK)
			{
			cout << "ERROR: Read position correction " << id << " failed.\n";
			delete pcor;
			return ERROR;
			}

		// append or replace
		if (append(pcors, pcor) == false)
			cout << "ATTENTION: Position correction " << id
				<< " was replaced.\n";
		}

	return OK;
}


// read attitude corrections

int read_att_correction(istream& is, vector<AttCorrection*>& acors)
{
	string id;

	while (is >> id)
		{
		if (id.compare("stop-dep") == 0) break;

		// create new object
		AttCorrection* acor = new AttCorrection;
		assert(acor != 0);

		// set ID
		acor->set_id(id);

		// read data
		if (acor->read(is) != OK)
			{
			cout << "ERROR: Read attitude correction " << id << " failed.\n";
			delete acor;
			return ERROR;
			}

		// append or replace
		if (append(acors, acor) == false)
			cout << "ATTENTION: Attitude correction " << id
				<< " was replaced.\n";
		}

	return OK;
}

} // namespace DEP
