/* $Id: lconeqn.cpp,v 1.2.2.2.2.1 2005/12/12 12:27:49 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "drvsub.h"
#include "lconeqn.h"
#include "cholesky.h"

namespace DEP
{

// constructor

LinConditionEqn::LinConditionEqn() : m_B(), _A(), _w(), _Qll() {}


// print linearized observation equations

void LinConditionEqn::print(ostream& os) const
{
	os << "design matrix B:\n";
	m_B.print(os);
	os << endl;

	os << "design submatrices Ai:\n";
	vector<DerivSub*>::const_iterator pb;
	for (pb = _A.begin(); pb != _A.end(); ++pb)
		(*pb)->print(os);

	os << "misclosure vector w:\n";
	_w.print(os);
	os << endl;

	os << "co-factor matrix Qll:\n";
	_Qll.print(os);
	os << endl;
}


// add contributions to the normal equations

bool LinConditionEqn::add_contr(Matrix<double>& AtPA, Vector<double>& AtPw,
double& wtPw) const
{
	// use improved vector/matrix operations and/or
	// LAPACK/BLAS routines for speed up!!!

	// weight matrix P
	Matrix<double> P;
	Matrix<double> BQ;
	//Matrix<double> BQBt;
	Vector<double> dec;

	// MS VC++ requires 4 parameters!
	mult(BQ, m_B, _Qll, true);
	mult_gen(P, true, BQ, false, m_B, true);

	int info = chol_dec(P, dec);
	if (info != 0)
		{
		cout << "ERROR: Decomposition of weight matrix failed.\n";
		return false;
		}

	chol_inv(P, dec);

	P.make_upper_symmetric();
	//cout << "P: " << P << endl;

	int i, j, ii, jj;

	// range-checking missing!!!
	// return ERROR;

	// calc. contribution to normal equation system
	vector<DerivSub*>::const_iterator d1;
	for (d1 = _A.begin(); d1 != _A.end(); ++d1)
		{
		//cout << "D1\n"; (*d1)->print(cout);
		Matrix<double> AtPi;
		mult_gen(AtPi, true, **d1, true, P, false);

		vector<DerivSub*>::const_iterator d2;
		for (d2 = d1; d2 != _A.end(); ++d2)
			{
			//cout << "D2\n"; (*d2)->print(cout);
			Matrix<double> AtPAi;

			// MS VC++ requires 4 parameters!
			mult(AtPAi, AtPi, **d2, true);
			//cout << "AtPAi: " << AtPAi << endl;

			// update normal equation matrix
			for (i = 0, ii = (*d1)->index(); i < AtPAi.rows(); i++, ii++)
				for (j = 0, jj = (*d2)->index(); j < AtPAi.cols(); j++, jj++)
					AtPA(ii,jj) += AtPAi(i,j);
			}

		Vector<double> AtPwi;
		mult(AtPwi, AtPi, _w);
		//cout << "AtPwi: " << AtPwi << endl;

		// update right-hand side
		for (i = 0, ii = (*d1)->index(); i < AtPwi.size(); i++, ii++)
			AtPw(ii) += AtPwi(i);
		}

	//wtpw += dot(P, w);
	Vector<double> wtP; wtP = mult(_w, P);
	wtPw += dot_prod(wtP, _w);

	return true;
}

} // namespace DEP
