/* $Id: params.cpp,v 1.1.1.1.4.1 2005/12/12 12:28:01 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "deperr.h"
#include "params.h"

namespace DEP
{

// constructor

Params::Params()
{
	// special options
	sray_opt = false;

	// output options
	pr_chk_appr = false;
	pr_chk_rays = false;
	pr_inter = false;
	pr_ic = false;

	// output options for adjustment objects
	pr_obj_crds = false;
	pr_ext_ori = false;
	pr_ap_sets = false;
	pr_pos_cor = false;
	pr_att_cor = false;

	// debug options
	db_upari = false;
	db_inter = false;

	// number of grid points in x and y direction
	pr_aps_nx = 5;
	pr_aps_ny = 5;

	// filenames of ...
	fname_normal_matrix_struct = "none";
	fname_normal_matrix = "none";
	fname_rhs_vector = "none";
	fname_solution_vector = "none";
}


// read parameter/options

int Params::read(istream& is)
{
	// options
	is >> sray_opt;
	if (!is)
		{
		cout << "Reading single-ray option failed.\n";
		return ERROR;
		}

	// output options
	is >> pr_chk_appr >> pr_chk_rays >> pr_inter >> pr_ic;
	if (!is)
		{
		cout << "Reading output flags failed.\n";
		return ERROR;
		}

	// output options for unknown parameters
	is >> pr_obj_crds >> pr_ext_ori >> pr_ap_sets >> pr_pos_cor >> pr_att_cor;
	if (!is)
		{
		cout << "Reading output flags for unknown parameters failed.\n";
		return ERROR;
		}

	// debug options
	is >> db_upari >> db_inter;
	if (!is)
		{
		cout << "Reading debug flags failed.\n";
		return ERROR;
		}

	// print of AP correction
	is >> pr_aps_nx >> pr_aps_ny;
	if (!is)
		{
		cout << "Reading output flags of AP corrections failed.\n";
		return ERROR;
		}

	// filename of normal matrix structure
	is >> fname_normal_matrix_struct;
	if (!is)
		{
		cout << "Reading filename of normal matrix structure failed.\n";
		return ERROR;
		}

	// filename of normal matrix
	is >> fname_normal_matrix;
	if (!is)
		{
		cout << "Reading filename of normal matrix failed.\n";
		return ERROR;
		}

	// filename of right hand side vector
	is >> fname_rhs_vector;
	if (!is)
		{
		cout << "Reading filename of right hand side vector failed.\n";
		return ERROR;
		}

	// filename of solution vector
	is >> fname_solution_vector;
	if (!is)
		{
		cout << "Reading filename of solution vector failed.\n";
		return ERROR;
		}

	return OK;
}


void Params::write(ostream& os) const
{
	os << sray_opt << endl;
	os << pr_chk_appr << " " << pr_chk_rays << " " << pr_inter << " "
		<< pr_ic << endl;
	os << pr_obj_crds << " " << pr_ext_ori << " " << pr_ap_sets << " "
		<< pr_pos_cor << " " << pr_att_cor;
	os << db_upari << " " << db_inter << endl;
	os << pr_aps_nx << " " << pr_aps_ny << endl;
	os << fname_normal_matrix_struct << endl;
	os << fname_normal_matrix << endl;
	os << fname_rhs_vector << endl;
	os << fname_solution_vector << endl;
}


void Params::print(ostream& os) const
{
	os << "Parameters/Options\n";
	os << "==================\n";
	os << "Single ray option                    :  "
		<< (sray_opt ? "on" : "off") << endl;

	os << "Print check approximations           :  "
		<< (pr_chk_appr ? "on" : "off") << endl;
	os << "Print check rays                     :  "
		<< (pr_chk_rays ? "on" : "off") << endl;
	os << "Print intersection results           :  "
		<< (pr_inter ? "on" : "off") << endl;
	os << "Print image coords.                  :  "
		<< (pr_ic ? "on" : "off") << endl;

	os << "Print unknown parameters\n";
	os << "  - object coords.                   :  "
		<< (pr_obj_crds ? "on" : "off") << endl;
	os << "  - exterior orientations            :  "
		<< (pr_ext_ori ? "on" : "off") << endl;
	os << "  - additional parameter sets        :  "
		<< (pr_ap_sets ? "on" : "off") << endl;
	os << "  - position corrections             :  "
		<< (pr_pos_cor ? "on" : "off") << endl;
	os << "  - attitude corrections             :  "
		<< (pr_att_cor ? "on" : "off") << endl;

	os << "Debug UPARI table                    :  "
		<< (db_upari ? "on" : "off") << endl;
	os << "Debug intersection                   :  "
		<< (db_inter ? "on" : "off") << endl;

	os << "Print AP set corrections\n";
	os << "  - number of grid points in x and y :  "
		<< pr_aps_nx << " x " << pr_aps_ny << endl;
	
	os << "Output of normal equations. Filename of ...\n";
	os << "  - normal matrix structure          :  "
		<< fname_normal_matrix_struct << endl;
	
	os << "  - normal matrix                    :  "
		<< fname_normal_matrix << endl;
	
	os << "  - right hand side vector           :  "
		<< fname_rhs_vector << endl;
	
	os << "  - solution vector                  :  "
		<< fname_solution_vector << endl;

	os << endl;
}

} // namespace DEP
