/* $Id: params.h,v 1.1.1.1.4.1 2005/12/12 12:28:01 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef DEP_PARAMS_INCLUDED
#define DEP_PARAMS_INCLUDED

#include "depdef.h"

namespace DEP
{

/** Parameters/options class.
 *  Parameters/options.
 *  @author Dirk Stallmann
 */
class Params
{
public:
	/// constructor
	Params();

	/// read
	int read(istream& is);

	/// write
	void write(ostream& os) const;

	/// print
	void print(ostream& os) const;

	/// single ray option
	bool	sray_opt;

	/// print check approximations
	bool	pr_chk_appr;

	/// print check rays
	bool	pr_chk_rays;

	/// print intersection results
	bool	pr_inter;

	/// print observed image coords.
	bool	pr_ic;

	/**@name print unknowns parameters during iterations.
	 * Output options for adjustment objects:
	 */
	//@{
		/// - object coords.
		bool	pr_obj_crds;

		/// - exterior orientation parameters
		bool	pr_ext_ori;

		/// - additional parameters
		bool	pr_ap_sets;

		/// - position corrections
		bool	pr_pos_cor;

		/// - attitude corrections
		bool	pr_att_cor;
	//@}

	// debug options:
	//@{
		/// print upari-table
		bool	db_upari;

		/// print intersection results
		bool	db_inter;
	//@}

	/// print effect of AP set.
	//@{
		/// number of grid points in x direction
		int		pr_aps_nx;

		/// number of grid points in y direction
		int		pr_aps_ny;
	//@}

	/// filename of normal matrix structure
	string	fname_normal_matrix_struct;

	/// filename of normal matrix
	string	fname_normal_matrix;

	/// filename of right hand side vector
	string	fname_rhs_vector;

	/// filename of solution vector
	string	fname_solution_vector;
};

} // namespace DEP

#endif
