/* $Id: pbcam.cpp,v 1.2.4.1 2005/12/12 12:28:01 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "deperr.h"
#include "pbcam.h"

namespace DEP
{

// constructor

PBCamera::PBCamera() : CameraCCD(), _x0(0.)
{
	_nr = 1; // only one row!!!
}


// read parameter

int PBCamera::read(istream& is)
{
	// identifier
	is >> _id;
	if (!is) return ERROR;

	// comment
	is >> _comment;
	if (!is) return ERROR;

	// focal length and principle point in mm
	is >> _focal_len >> _princ_pnt;
	if (!is) return ERROR;

	// convert from mm to m
	_princ_pnt *= 1e-3;
	_focal_len *= 1e-3;

	// number of pixel in line and pixel size
	is >> _nc >> _sc >> _sr;
	if (!is) return ERROR;

	// convert from um/pixel to m/pixel
	_sc *= 1e-6;
	_sr *= 1e-6;

	// CCD line position
	is >> _x0;

	_x0 *= 1e-3;	// convert from mm to m

	// calc. ICS location
	_c0 = (_nc - 1) / 2.;
	//_r0 = (_nr - 1) / 2.;

	// calc. image format
	_size_x = _nr * _sr;
	_size_y = _nc * _sc;

	return OK;
}


// write camera data

void PBCamera::write(ostream& os) const
{
	assert(false);
}


// print basic parameter

void PBCamera::print(ostream& os) const
{
	Form int10; int10.width(10);
	Form fix3(3); fix3.fixed().width(10);
	Form sci4(4); sci4.scientific().width(10);

	Camera::print(os);

	os << "Number of pixels   :  nc = " << int10(_nc) << "\n"; 
	os << "Pixel size         :  sc = " << fix3(_sc * 1e6) << " um/pix  "
		<< "sr = " << fix3(_sr * 1e6) << " um/pix\n";
	os << "CCD line position  :  x0 = " << fix3(_x0 * 1e3) << " mm\n"; 
	os << endl;
}


// get string of camera type 

string PBCamera::type_str() const
{
	ostringstream oss;
	oss << "pushbroom";
	return oss.str();
}


// external-to-image coordinates transformation

void PBCamera::external_to_image(const DPoint2& ext_c,
DPoint2& img_c) const
{
	img_c.x = _x0  + (ext_c.y - int(ext_c.y + 0.5)) * _sr;
	img_c.y = (ext_c.x - _c0) * _sc;
}


// image-to-pixel image coordinates transformation

void PBCamera::image_to_external(const DPoint2& img_c,
DPoint2& ext_c) const
{
	ext_c.x = img_c.y / _sc + _c0;
	ext_c.y = (img_c.x - _x0) / _sr; // number of lines are missing!?!
}


// ground sampling distance

double PBCamera::GSD(double hg) const
{
	return (hg * _sc / _focal_len);
}

} // namespace DEP
