/* $Id: pbcam.h,v 1.2.4.1 2005/12/12 12:28:01 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef DEP_PBCAM_INCLUDED
#define DEP_PBCAM_INCLUDED

#include "ccdcam.h"

using namespace std;
using namespace GenLib2;

namespace DEP
{

/** Pushbroom camera class.
 *  Implementation of an ''ideal'' pushbroom camera.
 *  Contains the inner orientation parameters of the imaging sensor.
 *  The scanning period per line or scan frequency is not defined here because
 *  the value might be variable for the camera!
 *
 *  @author Dirk Stallmann
 */
class PBCamera : public CameraCCD
{
public:
	/// constructor
	PBCamera();

	/// read
	virtual int read(istream& is);

	/// write
	virtual void write(ostream& os) const;
	
	/// print
	virtual void print(ostream& os) const;

	/// get camera type 
	virtual string type_str() const;

	/// get number of pixel per scanline
	//int np() const { return _np; }

	/**@name external/image coordinates transformation
	 *  To convert between external image (pixel) and
	 *  (internal) image coordinates.
	 */
	//@{
		/// external-to-image coordinates transformation
		virtual void external_to_image(const DPoint2& ext_c,
			DPoint2& img_c) const;

		/// image-to-external coordinates transformation
		virtual void image_to_external(const DPoint2& img_c,
			DPoint2& ext_c) const;
	//@}

	/// Ground Sampling Distance (GSD)
	double GSD(double hg) const;

private:
	/// CCD line position
	double	_x0;
};

} // namespace DEP

#endif
