/* $Id: pmflin.h,v 1.3.4.1 2005/12/12 12:28:01 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef DEP_PMFLIN_INCLUDED
#define DEP_PMFLIN_INCLUDED

#include "imggeo.h"

using namespace std;
using namespace GenLib2;

namespace DEP
{

/** Linear Polynomial Mapping Function (PMF) class.
 *  The linear PMF for the 8 unknown parameters a0, a1, a2, a3,
 *  b0, b1, b2, b3.
 *  @author Dirk Stallmann
 */
class PMFLinear : public ImageGeometry
{
public:
	/// constructor
	PMFLinear() : ImageGeometry(8) {}

	/**@name overridden base class AdjObject members */
	//@{
		/// get type name
		string type_str() const;

		/// get key name
		virtual string key_str() const;
	
		//// get parameter name
		string uname(size_t i) const;
	//@}

	/**@name overwrite pure virtual Image Geometry class members */
	//@{
		/// object-to-photo coords. transformation
		bool object_to_photo(const DPoint3& obj_c, DPoint3& pho_c) const;

		/// spatial intersection
		bool intersection(const ImageGeometry& g2,
			const DPoint3& pho_c1, const DPoint3& pho_c2,
			DPoint3& obj_c, double& e) const;

		/// get partial derivatives, pho_c are unused!
		bool get_derivs(const DPoint3& pho_c, const DPoint3& obj_c,
			DerivSub& deriv_oc, DerivSub& deriv_eo,
			DerivSub& deriv_pc, DerivSub& deriv_ac, DerivSub& deriv_obs);
	//@}

	/// spatial intersection
	bool intersection(const PMFLinear& pmf2,
		const DPoint3& pho_c1, const DPoint3& pho_c2,
		DPoint3& obj_c, double& e) const;
};


// non-member functions ...

/// projection using affine transformation
void affine_transform(const double* a, const DPoint3& obj_c, DPoint2& img_c);

/// intersection using affine transformation
bool affine_intersection(const double* a1, const DPoint2& pho_c1, 
const double* a2, const DPoint2& pho_c2, DPoint3& obj_c);


} // namespace DEP

#endif
