/* $Id: poscor.cpp,v 1.2.4.1 2005/12/12 12:28:01 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "deperr.h"
#include "depvec.h"
#include "poscor.h"
#include "extori.h"
#include "drvsub.h"

namespace DEP
{

// constructor

PosCorrection::PosCorrection() : _eo_ids(), _a(0), _sa(0), _t0(0.) {}


// read
// assume that all angles in degree

int PosCorrection::read(istream& is)
{
	// read EO IDs
	string eo_id;
	while (is >> eo_id && eo_id.compare("stop-dep") != 0)
		_eo_ids.push_back(eo_id);

	// reference time
	is >> _t0;
	
	// read number of coeffs.
	int sz = 0;
	is >> sz;
	_a = vector<DPoint3>(sz);
	_sa = vector<DPoint3>(sz);

	// read coeff. and std. devs.
	for (int i = 0; i < int(_a.size()); i++)
		{
		is >> _a[i].x >> _sa[i].x;
		is >> _a[i].y >> _sa[i].y;
		is >> _a[i].z >> _sa[i].z;
		}

	return OK;
}


// write
/*
void PosCorrection::write(ostream& os, SDevLim& sdev_lim)
{
	assert(_a.size() == _sa.size());

	os << id << "   " << name << endl;

	for (int i = 0; i < _a.size(); i++)
		{
		os << form("   %10.4e   %10.4e\n", _a[i].x, _sa[i].x);
		os << form("   %10.4e   %10.4e\n", _a[i].y, _sa[i].y);
		os << form("   %10.4e   %10.4e\n", _a[i].z, _sa[i].z);
		os << endl;
		}

	os << endl;
}
*/


// print

void PosCorrection::print(ostream& os) const
{
	assert(_a.size() == _sa.size());

	Form str4; str4.left().width(4);
	Form str8; str8.left().width(8);
	Form fix12(6); fix12.fixed().width(12);
	Form sci12(3); sci12.scientific().width(12);

	os << "Pos. corr. ID      :  " << _id << endl;
	os << "EOs using this set :  " << _eo_ids.size() << endl;
	int i = 0;
	for (i = 0; i < int(_eo_ids.size()); i++)
		{
		os << " " << _eo_ids[i];
		if ((i + 1) % 10 == 0) os << endl;
		}
	os << endl;
	os << "Reference time     :  t0 = " << fix12(_t0) << " s\n";
	os << endl;

	os << "  Param.     State          Value       Std.dev.\n\n";
	for (i = 0; i < int(_a.size()); i++)
		{
		os << "  " << str8(uname(i)) << "   " << str4(state_str(_sa[i].x))
			<< "   " << sci12(_a[i].x) << "   " << sci12(_sa[i].x) << endl;
		os << "  " << str8(uname(i)) << "   " << str4(state_str(_sa[i].y))
			<< "   " << sci12(_a[i].y) << "   " << sci12(_sa[i].y) << endl;
		os << "  " << str8(uname(i)) << "   " << str4(state_str(_sa[i].z))
			<< "   " << sci12(_a[i].z) << "   " << sci12(_sa[i].z) << endl;
		os << endl;
		}
	
	os << endl;
}


// get type name

string PosCorrection::type_str() const
{
	ostringstream oss;
	oss << "position correction";
	return oss.str();
}


// get key name

string PosCorrection::key_str() const
{
	ostringstream oss;
	oss << "pos-corr";
	return oss.str();
}


// get parameter name

string PosCorrection::uname(size_t i) const
{
	ostringstream oss;
	
	assert(i < 3 * _a.size());
	switch (i % 3)
		{
		case 0: oss << "X"; break;
		case 1: oss << "Y"; break;
		case 2: oss << "Z"; break;
		}
	oss << "o[" << (i / 3) << "]";

	return oss.str();
}


// set upari

void PosCorrection::set_upari()
{
	create(3 * _a.size());

	int k = 0;
	for (size_t i = 0; i < _a.size(); i++)
		{
	 	set(k, &_a[i].x, &_sa[i].x); k++;
	 	set(k, &_a[i].y, &_sa[i].y); k++;
	 	set(k, &_a[i].z, &_sa[i].z); k++;
		}
}


// print unknown parameters
/*
void PosCorrection::print_unknowns(ostream& os) const
{
	os << "Pos. corr. ID: " << _id << endl;
	os << endl;
	os << "  Param.     State         Value       Std.dev.    Ratio\n\n";

	for (int i = 0; i < _num; i++)
		{
		double t_abs = ratio(i); // test value
		os << form("  %-8s   %-4s   %12.3e   %12.3e   %6.1f",
			uname(i), state_cstr(i), par(i), sdev(i), t_abs);

		// test on significance
		if (is_eli(sdev(i)))
			os << "   is eliminated";
		else
			{
			if (chk_sign(t_abs))
				os << "   is significant";
			else
				os << "   not significant";
			}
		os << endl;	
		}
	
	os << endl;	
}
*/


// print changes of unknown parameters
/*
void PosCorrection::print_changes(ostream& os, const Vector<double>& dx) const
{
	//assert(_a.size() == _sa.size());

	os << "Pos. corr. ID    : " << _id << endl;
	os << endl;
	//os << "  Name       State"
	//	"        Value       Std.dev.    Ratio\n\n";

	int k = _index;
	for (int i = 0; i < _num; i++, k++)
		os << form("  %-8s   %-4s   %12.3e   %12.3e   %12.3e\n",
			uname(i), state_cstr(i), par(i), dx(k), (par(i) + dx(k)));
	os << endl;
}
*/


// get position correction
// evaluate polynomial p(dt) with the coefficients a[0..n]

DPoint3 PosCorrection::calc_corr(double t) const
{
	double dt = t - _t0;
    DPoint3 cor = _a[_a.size() - 1];

    for (int j = _a.size() - 2; j >= 0; j--)
        cor = cor * dt + _a[j];
	
	return cor;
}


// attach EO

int PosCorrection::attach(vector<ImageGeometry*>& eos)
{
	vector<string>::const_iterator id;
    for (id = _eo_ids.begin(); id != _eo_ids.end(); ++id)
		{
		ImageGeometry* p = find(eos, *id);
		if (p == 0)
			{
			cout << "ERROR: Position correction " << _id
				<< ": EO " << *id << " not found.\n";
			return ERROR;
			}

		// only useful for rotation angles
		if (ExtOrientation* eo = dynamic_cast<ExtOrientation*>(p))
			{
			eo->attach(this);
			this->inuse(true);
			}
		else
			{
			cout << "ERROR: Position correction " << _id
				<< ": EO " << *id << " invalid image geometry.\n";
			return ERROR;
			}
		}

	return OK;
}


// get derivatives

void PosCorrection::get_derivs(double t, const DerivSub& deriv_eo,
DerivSub& deriv_pc) const
{
	assert(deriv_eo.rows() == 2);
	assert(deriv_eo.cols() == 6);
	//cout << deriv_eo.rows() << ", " << deriv_eo.cols() << endl;

	int sz = 3 * _a.size();
	deriv_pc.index(_index);	// set index to normal equations
	deriv_pc.newsize(2, sz);

	deriv_pc(0,0) = deriv_eo(0,0);
	deriv_pc(1,0) = deriv_eo(1,0);
	deriv_pc(0,1) = deriv_eo(0,1);
	deriv_pc(1,1) = deriv_eo(1,1);
	deriv_pc(0,2) = deriv_eo(0,2);
	deriv_pc(1,2) = deriv_eo(1,2);

	double dt = t - _t0;

	for (int i = 3; i < sz; i++)
		{
		deriv_pc(0,i) = deriv_pc(0,i-3) * dt;
		deriv_pc(1,i) = deriv_pc(1,i-3) * dt;
		}
}

} // namespace DEP
